@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import enum
 import math
@@ -53,13 +48,7 @@ from hypothesis.control import cleanup, note
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.internal.cathetus import cathetus
 from hypothesis.internal.charmap import as_general_categories
-from hypothesis.internal.compat import (
-    ceil,
-    floor,
-    get_type_hints,
-    is_typed_named_tuple,
-    typing_root_type,
-)
+from hypothesis.internal.compat import ceil, floor, get_type_hints, is_typed_named_tuple
 from hypothesis.internal.conjecture.utils import (
     calc_label_from_cls,
     check_sample,
@@ -1001,16 +990,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
         # of it.  We do this in three places, hence the helper function
         if not isinstance(strat_or_callable, SearchStrategy):
             assert callable(strat_or_callable)  # Validated in register_type_strategy
-            try:
-                # On Python 3.6, typing.Hashable is just an alias for abc.Hashable,
-                # and the resolver function for Type throws an AttributeError because
-                # Hashable has no __args__.  We discard such errors when attempting
-                # to resolve subclasses, because the function was passed a weird arg.
-                strategy = strat_or_callable(thing)
-            except Exception:  # pragma: no cover
-                if not final:
-                    return nothing()
-                raise
+            strategy = strat_or_callable(thing)
         else:
             strategy = strat_or_callable
         if not isinstance(strategy, SearchStrategy):
@@ -1029,27 +1009,11 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
             if thing in types._global_type_lookup:
                 return as_strategy(types._global_type_lookup[thing], thing)
             return from_type(thing.__supertype__)
-        # Under Python 3.6, Unions are not instances of `type` - but we
-        # still want to resolve them!
+        # Unions are not instances of `type` - but we still want to resolve them!
         if getattr(thing, "__origin__", None) is typing.Union:
             args = sorted(thing.__args__, key=types.type_sorting_key)
             return one_of([from_type(t) for t in args])
     if not types.is_a_type(thing):
-        # The implementation of typing_extensions.Literal under Python 3.6 is
-        # *very strange*.  Notably, `type(Literal[x]) != Literal` so we have to
-        # use the first form directly, and because it uses __values__ instead of
-        # __args__ we inline the relevant logic here until the end of life date.
-        if types.is_typing_literal(thing):  # pragma: no cover
-            assert sys.version_info[:2] == (3, 6)
-            args_dfs_stack = list(thing.__values__)  # type: ignore
-            literals = []
-            while args_dfs_stack:
-                arg = args_dfs_stack.pop()
-                if types.is_typing_literal(arg):
-                    args_dfs_stack.extend(reversed(arg.__values__))
-                else:
-                    literals.append(arg)
-            return sampled_from(literals)
         if isinstance(thing, str):
             # See https://github.com/HypothesisWorks/hypothesis/issues/3016
             raise InvalidArgument(
@@ -1059,8 +1023,20 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
                 "strings."
             )
         raise InvalidArgument(f"thing={thing!r} must be a type")  # pragma: no cover
+    if thing in types.TypeAliasTypes:
+        # Code like `st.from_type(TypeAlias)` does not make sense.
+        raise InvalidArgument(
+            "Cannot resolve TypeAlias to a strategy, "
+            "because there are no instances of it at runtime"
+        )
+    if thing in types.ClassVarTypes:
+        # Code like `st.from_type(ClassVar)` does not make sense.
+        raise InvalidArgument(
+            "Cannot resolve ClassVar to a strategy, "
+            "because there are no instances of it at runtime"
+        )
     # Now that we know `thing` is a type, the first step is to check for an
-    # explicitly registered strategy.  This is the best (and hopefully most
+    # explicitly registered strategy. This is the best (and hopefully most
     # common) way to resolve a type to a strategy.  Note that the value in the
     # lookup may be a strategy or a function from type -> strategy; and we
     # convert empty results into an explicit error.
@@ -1097,7 +1073,7 @@ def _from_type(thing: Type[Ex]) -> SearchStrategy[Ex]:
     # We'll start by checking if thing is from from the typing module,
     # because there are several special cases that don't play well with
     # subclass and instance checks.
-    if isinstance(thing, typing_root_type) or (
+    if isinstance(thing, types.typing_root_type) or (
         sys.version_info[:2] >= (3, 9)
         and isinstance(getattr(thing, "__origin__", None), type)
         and getattr(thing, "__args__", None)
@@ -1787,6 +1763,16 @@ def register_type_strategy(
 
     if not types.is_a_type(custom_type):
         raise InvalidArgument(f"custom_type={custom_type!r} must be a type")
+    elif custom_type in types.TypeAliasTypes:
+        raise InvalidArgument(
+            f"custom_type={custom_type!r} is not allowed to be registered, "
+            "because there is no such thing as a runtime instance of TypeAlias"
+        )
+    elif custom_type in types.ClassVarTypes:
+        raise InvalidArgument(
+            f"custom_type={custom_type!r} is not allowed to be registered, "
+            "because there is no such thing as a runtime instance of ClassVar"
+        )
     elif not (isinstance(strategy, SearchStrategy) or callable(strategy)):
         raise InvalidArgument(
             "strategy=%r must be a SearchStrategy, or a function that takes "
